"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurationService = void 0;
const enphase_data_service_1 = require("../enphase/enphase.data.service");
const tesla_service_1 = require("../tesla/tesla.service");
const fs = require("fs");
class ConfigurationService {
    static init() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Init of ConfigurationService');
            // Read configuration file
            if (fs.existsSync('./assets/conf/config.json')) {
                console.log('Config file is here. Read the config file to retreave it.');
                let str = fs.readFileSync('./assets/conf/config.json', 'utf8');
                ConfigurationService.configuration = JSON.parse(str);
                console.log('Configuration file loaded');
            }
            else {
                console.log('Config file not here. Read the default config to create it.');
                let default_str = fs.readFileSync('./assets/conf/default_config.json', 'utf8');
                ConfigurationService.configuration = JSON.parse(default_str);
                fs.writeFileSync('./assets/conf/config.json', JSON.stringify(ConfigurationService.configuration, null, 2), 'utf8');
                console.log('Config created from default.');
            }
            console.log('End of Init of ConfigurationService');
        });
    }
    static getConfiguration() {
        return ConfigurationService.configuration;
    }
    static setConfiguration(conf, attrib, from) {
        // Do not change token, user etc..., password if empty
        if (conf.enphase.token === '') {
            conf.enphase.token = ConfigurationService.configuration.enphase.token;
        }
        if (conf.enphase.user === '') {
            conf.enphase.user = ConfigurationService.configuration.enphase.user;
        }
        if (conf.enphase.password === '') {
            conf.enphase.password = ConfigurationService.configuration.enphase.password;
        }
        if (conf.tesla.token === '') {
            // Au lieu d'utiliser la version en mémoire, lisons le fichier pour avoir le token le plus récent
            try {
                const configFile = JSON.parse(fs.readFileSync('./assets/conf/config.json', 'utf8'));
                if (configFile.tesla && configFile.tesla.token) {
                    conf.tesla.token = configFile.tesla.token;
                    // Synchroniser aussi notre configuration en mémoire
                    ConfigurationService.configuration.tesla.token = configFile.tesla.token;
                } else {
                    conf.tesla.token = ConfigurationService.configuration.tesla.token;
                }
            } catch (e) {
                // En cas d'erreur, utiliser la valeur en mémoire comme fallback
                conf.tesla.token = ConfigurationService.configuration.tesla.token;
                console.log('Error reading Tesla token from file:', e.message);
            }
        }
        if (!conf.tesla.BLEexecutablePath) {
            conf.tesla.BLEexecutablePath = ConfigurationService.configuration.tesla.BLEexecutablePath || "/home/jlo/go/bin/tesla-control";
            console.log('Preserving BLEexecutablePath:', conf.tesla.BLEexecutablePath);
        }
        
        if (!conf.tesla.BLECommandDelay && ConfigurationService.configuration.tesla.BLECommandDelay) {
            conf.tesla.BLECommandDelay = ConfigurationService.configuration.tesla.BLECommandDelay;
            console.log('Preserving BLECommandDelay:', conf.tesla.BLECommandDelay);
        }
        
        if (!conf.tesla.BLECommandTimeout && ConfigurationService.configuration.tesla.BLECommandTimeout) {
            conf.tesla.BLECommandTimeout = ConfigurationService.configuration.tesla.BLECommandTimeout;
        }
        
        if (!conf.tesla.BLEWakeupWaitTime && ConfigurationService.configuration.tesla.BLEWakeupWaitTime) {
            conf.tesla.BLEWakeupWaitTime = ConfigurationService.configuration.tesla.BLEWakeupWaitTime;
        }
        if (conf.application.pwd === '') {
            conf.application.pwd = ConfigurationService.configuration.application.pwd;
        }
        if (conf.enphase.userEnphaseEnergy === '') {
            conf.enphase.userEnphaseEnergy = ConfigurationService.configuration.enphase.userEnphaseEnergy;
        }
        if (conf.enphase.passwordEnphaseEnergy === '') {
            conf.enphase.passwordEnphaseEnergy = ConfigurationService.configuration.enphase.passwordEnphaseEnergy;
        }
        if (conf.enphase.gatewaySerial === '') {
            conf.enphase.gatewaySerial = ConfigurationService.configuration.enphase.gatewaySerial;
        }
        ConfigurationService.configuration = conf;
        enphase_data_service_1.EnphaseDataService.setConfiguration(conf, attrib, from);
        tesla_service_1.TeslaService.setConfiguration(conf, attrib, from);
        fs.writeFileSync('./assets/conf/config.json', JSON.stringify(conf, null, 2), 'utf8');
    }
}
exports.ConfigurationService = ConfigurationService;
