"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tesla_service_1 = require("./tesla.service");
const configuration_service_1 = require("../services/configuration.service");
// getting last vehicle data
const getLastData = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        return res.status(400).json({ error: 'can\'t get tesla data because configuration useTesla parameter is false' });
    }
    let result = {};
    try {
        result = tesla_service_1.TeslaService.getLastReceiveData();
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get tesla data' });
    }
    return res.status(200).json(result);
});
// getting a refresh
const getRefresh = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        return res.status(400).json({ error: 'can\'t get tesla data because configuration useTesla parameter is false' });
    }
    let result = {};
    try {
        result = tesla_service_1.TeslaService.getImmediateVehicleData();
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t get tesla refresh' });
    }
    return res.status(200).json(result);
});
// setting a mode
const getMode = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        return res.status(400).json({ error: 'can\'t get tesla data because configuration useTesla parameter is false' });
    }
    let result = {};
    try {
        if (!req.query.chargingMode) {
            return res.status(400).json({ error: 'param chargingMode not found' });
        }
        // set charging mode
        let chargingMode = req.query.chargingMode;
        result = tesla_service_1.TeslaService.setMode(chargingMode);
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t set tesla charging mode' });
    }
    return res.status(200).json(result);
});
// setting a climate
const getClimate = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        return res.status(400).json({ error: 'can\'t set climate because configuration useTesla parameter is false' });
    }
    let result = {};
    try {
        if (!req.query.state) {
            return res.status(400).json({ error: 'param state not found' });
        }
        // set climate state
        let state = req.query.state.toUpperCase() === 'TRUE';
        if (state) {
            result = tesla_service_1.TeslaService.startClimatisation();
        }
        else {
            result = tesla_service_1.TeslaService.stopClimatisation();
        }
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t set tesla climatisation' });
    }
    return res.status(200).json(result);
});
// starting or stopping charge
const getCharge = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    if (!configuration_service_1.ConfigurationService.getConfiguration().tesla.useTesla) {
        return res.status(400).json({ error: 'can\'t set charge state because configuration useTesla parameter is false' });
    }
    let result = false;
    try {
        if (!req.query.state) {
            return res.status(400).json({ error: 'param state not found' });
        }
        // set charging state
        let state = req.query.state.toUpperCase() === 'TRUE';
        if (state) {
            result = yield tesla_service_1.TeslaService.startCharge();
        }
        else {
            result = yield tesla_service_1.TeslaService.stopCharge();
        }
    }
    catch (e) {
        return res.status(500).json({ error: 'can\'t set tesla charge state' });
    }
    return res.status(200).json({ result: result });
});
exports.default = {
    getLastData,
    getRefresh,
    getMode,
    getClimate,
    getCharge
};
